@@ -18,30 +18,37 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with qpsolvers. If not, see <http://www.gnu.org/licenses/>.
 
+"""Solver interface for MOSEK"""
+
+from typing import Optional
+
 import cvxopt.msk
 import mosek
 
+from numpy import ndarray
+
 from .cvxopt_ import cvxopt_solve_qp
 
-cvxopt.solvers.options['mosek'] = {mosek.iparam.log: 0}
 
-
-def mosek_set_verbosity(verbose):
-    cvxopt.solvers.options['mosek'] = {mosek.iparam.log: 1 if verbose else 0}
-
-
-def mosek_solve_qp(P, q, G, h, A=None, b=None, initvals=None):
+def mosek_solve_qp(
+    P, q, G, h, A=None, b=None, initvals=None, verbose: bool = False
+) -> Optional[ndarray]:
     """
     Solve a Quadratic Program defined as:
 
-        minimize
-            (1/2) * x.T * P * x + q.T * x
+    .. math::
 
-        subject to
-            G * x <= h
-            A * x == b
+        \\begin{split}\\begin{array}{ll}
+        \\mbox{minimize} &
+            \\frac{1}{2} x^T P x + q^T x \\\\
+        \\mbox{subject to}
+            & G x \\leq h                \\\\
+            & A x = h                    \\\\
+            & lb \\leq x \\leq ub
+        \\end{array}\\end{split}
 
-    using CVXOPT <http://cvxopt.org/>.
+    using the `MOSEK interface from CVXOPT
+    <https://cvxopt.org/userguide/coneprog.html#optional-solvers>`_.
 
     Parameters
     ----------
@@ -59,10 +66,13 @@ def mosek_solve_qp(P, q, G, h, A=None, b=None, initvals=None):
         Linear equality constraint vector.
     initvals : numpy.array, optional
         Warm-start guess vector.
+    verbose : bool, optional
+        Set to `True` to print out extra information.
 
     Returns
     -------
     x : numpy.array
         Solution to the QP, if found, otherwise ``None``.
     """
-    return cvxopt_solve_qp(P, q, G, h, A, b, 'mosek', initvals)
+    cvxopt.solvers.options["mosek"] = {mosek.iparam.log: 1 if verbose else 0}
+    return cvxopt_solve_qp(P, q, G, h, A, b, "mosek", initvals)
