@@ -18,7 +18,11 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with qpsolvers. If not, see <http://www.gnu.org/licenses/>.
 
-from numpy import array, hstack, ones, vstack, zeros
+"""Solver interface for qpOASES"""
+
+from typing import Optional
+
+from numpy import array, hstack, ndarray, ones, vstack, zeros
 from qpoases import PyOptions as Options
 from qpoases import PyPrintLevel as PrintLevel
 from qpoases import PyQProblem as QProblem
@@ -26,28 +30,36 @@ from qpoases import PyQProblemB as QProblemB
 from qpoases import PyReturnValue as ReturnValue
 
 
-__infty = 1e10
-options = Options()
-options.printLevel = PrintLevel.NONE
+__infty__ = 1e10
+__options__ = Options()
+__options__.printLevel = PrintLevel.NONE
 
 
-def qpoases_set_verbosity(verbose):
-    options.printLevel = PrintLevel.MEDIUM if verbose else PrintLevel.NONE
-
-
-def qpoases_solve_qp(P, q, G=None, h=None, A=None, b=None, initvals=None,
-                     max_wsr=1000):
+def qpoases_solve_qp(
+    P,
+    q,
+    G=None,
+    h=None,
+    A=None,
+    b=None,
+    initvals=None,
+    verbose: bool = False,
+    max_wsr: int = 1000,
+) -> Optional[ndarray]:
     """
     Solve a Quadratic Program defined as:
 
-        minimize
-            (1/2) * x.T * P * x + q.T * x
+    .. math::
 
-        subject to
-            G * x <= h
-            A * x == b
+        \\begin{split}\\begin{array}{ll}
+        \\mbox{minimize} &
+            \\frac{1}{2} x^T P x + q^T x \\\\
+        \\mbox{subject to}
+            & G x \\leq h                \\\\
+            & A x = h
+        \\end{array}\\end{split}
 
-    using qpOASES <https://projects.coin-or.org/qpOASES>.
+    using `qpOASES <https://projects.coin-or.org/qpOASES>`_.
 
     Parameters
     ----------
@@ -65,7 +77,9 @@ def qpoases_solve_qp(P, q, G=None, h=None, A=None, b=None, initvals=None,
         Linear equality constraint vector.
     initvals : numpy.array, optional
         Warm-start guess vector.
-    max_wsr : integer, optional
+    verbose : bool, optional
+        Set to `True` to print out extra information.
+    max_wsr : int, optional
         Maximum number of Working-Set Recalculations given to qpOASES.
 
     Returns
@@ -103,17 +117,18 @@ def qpoases_solve_qp(P, q, G=None, h=None, A=None, b=None, initvals=None,
         ub_C = b
     elif G is not None and A is not None:
         C = vstack([G, A, A])
-        lb_C = hstack([-__infty * ones(h.shape[0]), b, b])
+        lb_C = hstack([-__infty__ * ones(h.shape[0]), b, b])
         ub_C = hstack([h, b, b])
+    __options__.printLevel = PrintLevel.MEDIUM if verbose else PrintLevel.NONE
     if has_cons:
         qp = QProblem(n, C.shape[0])
-        qp.setOptions(options)
+        qp.setOptions(__options__)
         return_value = qp.init(P, q, C, lb, ub, lb_C, ub_C, array([max_wsr]))
         if return_value == ReturnValue.MAX_NWSR_REACHED:
             print("qpOASES reached the maximum number of WSR (%d)" % max_wsr)
     else:
         qp = QProblemB(n)
-        qp.setOptions(options)
+        qp.setOptions(__options__)
         qp.init(P, q, lb, ub, max_wsr)
     x_opt = zeros(n)
     ret = qp.getPrimalSolution(x_opt)
